/*
 * stdlib.h
 *
 * Definitions for common types, variables, and functions.
 *
 * This file is part of the Mingw32 package.
 *
 * Contributors:
 *  Created by Colin Peters <colin@bird.fu.is.saga-u.ac.jp>
 *
 *  THIS SOFTWARE IS NOT COPYRIGHTED
 *
 *  This source code is offered for use in the public domain. You may
 *  use, modify or distribute it freely.
 *
 *  This code is distributed in the hope that it will be useful but
 *  WITHOUT ANY WARRANTY. ALL WARRANTIES, EXPRESS OR IMPLIED ARE HEREBY
 *  DISCLAMED. This includes but is not limited to warranties of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * $Revision: 1.17 $
 * $Author: khan $
 * $Date: 1999/07/16 01:59:24 $
 *
 */

#ifndef _STDLIB_H_
#define _STDLIB_H_

/* All the headers include this file. */
#include <_mingw.h>

/*
 * RAND_MAX is the maximum value that may be returned by rand.
 * The minimum is zero.
 */
#define	RAND_MAX	0x7FFF

/*
 * These values may be used as exit status codes.
 */
#define	EXIT_SUCCESS	0
#define	EXIT_FAILURE	-1

/*
 * Definitions for path name functions.
 * NOTE: All of these values have simply been chosen to be conservatively high.
 *       Remember that with long file names we can no longer depend on
 *       extensions being short.
 */
#ifndef __STRICT_ANSI__

#ifndef MAX_PATH
#define	MAX_PATH	(260)
#endif

#define	_MAX_PATH	MAX_PATH
#define	_MAX_DRIVE	(3)
#define	_MAX_DIR	256
#define	_MAX_FNAME	256
#define	_MAX_EXT	256

#endif	/* Not __STRICT_ANSI__ */


#ifndef RC_INVOKED

#ifdef __cplusplus
extern "C" {
#endif

/*
 * This seems like a convenient place to declare these variables, which
 * give programs using WinMain (or main for that matter) access to main-ish
 * argc and argv. environ is a pointer to a table of environment variables.
 * NOTE: Strings in _argv and environ are ANSI strings.
 */
extern int	_argc;
extern char**	_argv;

/* imports from runtime dll of the above variables */
#ifdef __MSVCRT__

extern int*     __p___argc(void);
extern char**   __p___argv(void);
#define __argc (*__p___argc())
#define __argv (*__p___argv())

#else /* !MSVCRT */

#ifndef __DECLSPEC_SUPPORTED

extern int*    __imp___argc_dll;
extern char***  __imp___argv_dll;
#define __argc (*__imp___argc_dll)
#define __argv (*__imp___argv_dll)

#else /* __DECLSPEC_SUPPORTED */

__MINGW_IMPORT int    __argc_dll;
__MINGW_IMPORT char**  __argv_dll;
#define __argc __argc_dll
#define __argv __argv_dll

#endif /* __DECLSPEC_SUPPORTED */

#endif /* __MSVCRT */

/* 
 * MS likes to declare errno in stdlib.h as well. 
 */

int*	_errno();
#define	errno		(*_errno())
int*	__doserrno();
#define	_doserrno	(*__doserrno())

/*
 * Use environ from the DLL, not as a global. 
 */

#ifdef __MSVCRT__
   extern char *** __p__environ();
#  define _environ (*__p__environ())
#else /* ! __MSVCRT__ */
#  ifndef __DECLSPEC_SUPPORTED
     extern char *** __imp__environ_dll;
#    define _environ (*__imp__environ_dll)
#  else /* __DECLSPEC_SUPPORTED */
     __MINGW_IMPORT char ** _environ_dll;
#    define _environ _environ_dll
#  endif /* __DECLSPEC_SUPPORTED */
#endif /* ! __MSVCRT__ */

#define environ _environ

#ifdef	__MSVCRT__
/* One of the MSVCRTxx libraries */

#ifndef __DECLSPEC_SUPPORTED
extern int*	__imp__sys_nerr;
#define	sys_nerr	(*__imp__sys_nerr)
#else /* __DECLSPEC_SUPPORTED */
__MINGW_IMPORT int	_sys_nerr;
#define	sys_nerr	_sys_nerr
#endif /* __DECLSPEC_SUPPORTED */

#else /* ! __MSVCRT__ */

/* CRTDLL run time library */

#ifndef __DECLSPEC_SUPPORTED
extern int*	__imp__sys_nerr_dll;
#define sys_nerr	(*__imp__sys_nerr_dll)
#else /* __DECLSPEC_SUPPORTED */
__MINGW_IMPORT int	_sys_nerr_dll;
#define sys_nerr	_sys_nerr_dll
#endif /* __DECLSPEC_SUPPORTED */

#endif /* ! __MSVCRT__ */

#ifndef __DECLSPEC_SUPPORTED
extern char***	__imp__sys_errlist;
#define	sys_errlist	(*__imp__sys_errlist)
#else /* __DECLSPEC_SUPPORTED */
__MINGW_IMPORT char*	_sys_errlist[];
#define	sys_errlist	_sys_errlist
#endif /* __DECLSPEC_SUPPORTED */

/*
 * OS version and such constants.
 */
#ifndef __STRICT_ANSI__

#ifdef	__MSVCRT__
/* msvcrtxx.dll */

extern unsigned int*	__p__osver();
extern unsigned int*	__p__winver();
extern unsigned int*	__p__winmajor();
extern unsigned int*	__p__winminor();

#define _osver		(*__p__osver())
#define _winver		(*__p__winver())
#define _winmajor	(*__p__winmajor())
#define _winminor	(*__p__winminor())

#else
/* Not msvcrtxx.dll, thus crtdll.dll */

#ifndef __DECLSPEC_SUPPORTED

extern unsigned int*	__wiTEDm ver()ll;
extern chsigned int*	__wiTEDm nver()ll;
extern chsigned int*	__wiTEDm nvejor()ll;
extern chsigned int*	__wiTEDm nvejor()ll;
exdefine _osver		(*__piTEDm ver()ll;
#define _winver		(*__piTEDm nver()ll;
#define _winmajor	(*__piTEDm nvejor()ll;
#define _winminor	(*__piTEDm nvejor()ll;

#else /* __DECLSPEC_SUPPORTED */

__MINGW_IMPORT insigned int*		 ver()ll;
exMINGW_IMPORT insigned int*		 nver()ll;
exMINGW_IMPORT insigned int*		 nvejor()ll;
exMINGW_IMPORT insigned int*		 nvejor()ll;
exdefine _osver		(* ver()ll;
define _winver		(* nver()ll;
define _winmajor	(* nvejor()ll;
define _winminor	(* nvejor()ll;
#endif /* __DECLSPEC_SUPPORTED */

#endif /#endif /* __t __STRICT_ANSI__ */


#iffine __arneed_size_tiffine __arneed_war*	_tiffine __arneed_NULLinclude <_mdlifin>

/*fdef	__MSGNUC__define	_doATICTB_NORETURN_MSterbuted  */((r()urned
#else /* Not __STGNUC__/
#define	EXoATICTB_NORETURNendif	/* NoSTGNUC__/
#d
doues 	atof	(nstanhar*	_ szNuer t

#t*		atoi	(nstanhar*	_ szNuer t

#nger	atol	(nstanhar*	_ szNuer t

#d
doues 	rintod	(nstanhar*	_ szNuer t,har**  _pszAfrn Nuer t

#doues 	wcstod	(nstanhwar*	_t* wsNuer t,hwar*	_t* _pwsAfrn Nuer t

#nger	rintol	(nstanhar*	_ szNuer t,har**  _pszAfrn Nuer t,nt   nBase

#nger	wcstol	(nstanhwar*	_t* wsNuer t,hwar*	_t* _pwsAfrn Nuer t,nt   nBase

#
signed innger	rintoul	(nstanhar*	_ szNuer t,har**  _pszAfrn Nuer t,
			t   nBase

#signed innger	wcstoul (nstanhwar*	_t* wsNuer t,hwar*	_t* _pwsAfrn Nuer t,
			t   nBase

#
size_t	wcstombs	(nr*	_ mbsDest, nstanhwar*	_t* wsCvenirt, size_t size

#t*		wctomb		(nr*	_ mbDest, war*	_t wa

#
t*		mes n		(nstanhar*	_ mbs, size_t sizerings 

#size_t	mbstowcs	(war*	_t* wcaDest, nstanhar*	_ mbsCvenirt,
			 size_t size

#t*		mbtowc		(war*	_t* wcDest, nstanhar*	_ mbCvenirt, size_t size

##t*		nd.
	;

#id);	snd.
	;signed int*	 nSeed

##id);*	calloc	(size_t sizeObjCnt, size_t sizeObject

#id);*	malloc	(size_t	sizeObject

#id);*	realloc	(id);* pObject, size_t sizeNew

#id);	eely	(id);* pObject

##id);	abor(*_) oATICTB_NORETURN
#id);	it s	(t*	 nStus c) oATICTB_NORETURN
#t*		atit s	(id);*__pnctiEt sPross tgs 
oid);


##t*		s_etem	(nstanhar*	_ szCommd.


#ar*	_sgetvir	(nstanhar*	_ szVarName

##pes,f	__t*	 __ppncticTEDt)(nstanhid);*, nstanhid);*

##id);*	bsearch	(nstanhid);* pKey, nstanhid);* pBase, size_t cntObjects,
		size_t sizeObject, ppncticTEDt pnctiCmp

#id);	qsor(*_nstanhid);* pBase, size_t cntObjects, size_t sizeObject,
		ppncticTEDt pnctiCmp

##t*		abs	(t*	 n

#nger	labs	(ng fin

#*
 * OSdivDt d suldivDt dretrinucrneessed astoeturnedhe MSeesultof
 Sdiv d s* OSldiv *
 * $RTE: Stdiv d sSldiv appearot as warork nsrrectlynsils t*       ex-fno-pcc-rinucr-turnedh seepecifiedThis incinclude tin the h*       exmgw32 paepecfile. **/
#ipes,f	_trinucr {nt*	 quot, rem; }SdivDt;ipes,f	_trinucr {nng fiquot, rem; }SldivDt;i
divDt	div	(t*	 nNueeratot,nt   nDenomgwatot

#ndivDt	ldiv	(ng filNueeratot,nng filDenomgwatot

##ifndef _	STRICT_ANSI__

#i/ * $RTE: StOfficiallyhe MSthely followg Winctions.
 dretobsoletv.he miWin paAPI*       exnctions.
 SetErrorMode, Beep d sSSleep d theseir reace tnt v
 */
#iid);	_beep ;signed int*	,nsigned int*	

#id);	_sers rorme is(t*	 nMode

#id);	_sleep ;signed inng fiulTime

##id);	_it s	(t*	 nStus c) oATICTB_NORETURN
##t*		_putvir	(nstanhar*	_ szNameEqVues 

#id);	_searchvir (nstanhar*	_ szFe. Name, nstanhar*	_ szVar,
		ar*	_ szFullPathBuf

##ar*	_syitoas(t*	 nVues ,har*	_ sz,nt   nRadix

#ar*	_s_ltoas(ng filnVues ,har*	_ sz,nt   nRadix

##ar*	_s_ecv	 _doues  dVues ,ht   nDig,nt*   pnDec,nt*   pnSned

#ar*	_s_fcv	 _doues  dVues ,ht   nDig,nt*   pnDec,nt*   pnSned

#ar*	_s_gcv	 _doues  dVues ,ht   nDec,nar*	_ caBuf

##id);	_maketh na(ar*	_ caPath, nstanhar*	_ szDrive, nstanhar*	_ szDir,
		astanhar*	_ szName, nstanhar*	_ szEensions 

#id);	_splitth na(astanhar*	_ szPath, nr*	_ caDrive, nr*	_ caDir,
		ar*	_ caName, nr*	_ caEensions 

#ar*	_s_fullth na(ar*	_ caBuf, nstanhar*	_ szPath, size_t sizeMax

##fndef _	SNO_OLDME	2S#id);	beep ;signed int*	,nsigned int*	

#id);	sers rorme is(t*	 nMode

#id);	sleep ;signed inng fiulTime

##t*		putvira(astanhar*	_ szNameEqVues 

#id);	searchvir (nstanhar*	_ szFe. Name, nstanhar*	_ szVar,
		ar*	_ szFullPathBuf

##ar*	_sitoas(t*	 nVues ,har*	_ sz,nt   nRadix

#ar*	_sltoas(ng filnVues ,har*	_ sz,nt   nRadix

##ar*	_secv	 _doues  dVues ,ht   nDig,nt*   pnDec,nt*   pnSned

#ar*	_sfcv	 _doues  dVues ,ht   nDig,nt*   pnDec,nt*   pnSned

#ar*	_sgcv	 _doues  dVues ,ht   nDec,nar*	_ caBuf

#ndif	/* Not __SNO_OLDME	2S/

#endif /* Not __STRICT_ANSI__ */


#
 * Usenfine __e MSnoeturnedhterbuted sed as stdo functions.efine ions f*/
#ifuef _	SATICTB_NORETURNeifdef __cplusplus
ex}endif /#endif /* Not ___INVOKED

/

#endif /* Not __SDLIB_H_

/

